@@ -27,7 +27,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include "redis.h"
+#include "server.h"
 
 /* This file implements keyspace events notification via Pub/Sub ad
  * described at http://redis.io/topics/keyspace-events. */
@@ -43,17 +43,17 @@ int keyspaceEventsStringToFlags(char *classes) {
 
     while((c = *p++) != '\0') {
         switch(c) {
-        case 'A': flags |= REDIS_NOTIFY_ALL; break;
-        case 'g': flags |= REDIS_NOTIFY_GENERIC; break;
-        case '$': flags |= REDIS_NOTIFY_STRING; break;
-        case 'l': flags |= REDIS_NOTIFY_LIST; break;
-        case 's': flags |= REDIS_NOTIFY_SET; break;
-        case 'h': flags |= REDIS_NOTIFY_HASH; break;
-        case 'z': flags |= REDIS_NOTIFY_ZSET; break;
-        case 'x': flags |= REDIS_NOTIFY_EXPIRED; break;
-        case 'e': flags |= REDIS_NOTIFY_EVICTED; break;
-        case 'K': flags |= REDIS_NOTIFY_KEYSPACE; break;
-        case 'E': flags |= REDIS_NOTIFY_KEYEVENT; break;
+        case 'A': flags |= NOTIFY_ALL; break;
+        case 'g': flags |= NOTIFY_GENERIC; break;
+        case '$': flags |= NOTIFY_STRING; break;
+        case 'l': flags |= NOTIFY_LIST; break;
+        case 's': flags |= NOTIFY_SET; break;
+        case 'h': flags |= NOTIFY_HASH; break;
+        case 'z': flags |= NOTIFY_ZSET; break;
+        case 'x': flags |= NOTIFY_EXPIRED; break;
+        case 'e': flags |= NOTIFY_EVICTED; break;
+        case 'K': flags |= NOTIFY_KEYSPACE; break;
+        case 'E': flags |= NOTIFY_KEYEVENT; break;
         default: return -1;
         }
     }
@@ -68,20 +68,20 @@ sds keyspaceEventsFlagsToString(int flags) {
     sds res;
 
     res = sdsempty();
-    if ((flags & REDIS_NOTIFY_ALL) == REDIS_NOTIFY_ALL) {
+    if ((flags & NOTIFY_ALL) == NOTIFY_ALL) {
         res = sdscatlen(res,"A",1);
     } else {
-        if (flags & REDIS_NOTIFY_GENERIC) res = sdscatlen(res,"g",1);
-        if (flags & REDIS_NOTIFY_STRING) res = sdscatlen(res,"$",1);
-        if (flags & REDIS_NOTIFY_LIST) res = sdscatlen(res,"l",1);
-        if (flags & REDIS_NOTIFY_SET) res = sdscatlen(res,"s",1);
-        if (flags & REDIS_NOTIFY_HASH) res = sdscatlen(res,"h",1);
-        if (flags & REDIS_NOTIFY_ZSET) res = sdscatlen(res,"z",1);
-        if (flags & REDIS_NOTIFY_EXPIRED) res = sdscatlen(res,"x",1);
-        if (flags & REDIS_NOTIFY_EVICTED) res = sdscatlen(res,"e",1);
+        if (flags & NOTIFY_GENERIC) res = sdscatlen(res,"g",1);
+        if (flags & NOTIFY_STRING) res = sdscatlen(res,"$",1);
+        if (flags & NOTIFY_LIST) res = sdscatlen(res,"l",1);
+        if (flags & NOTIFY_SET) res = sdscatlen(res,"s",1);
+        if (flags & NOTIFY_HASH) res = sdscatlen(res,"h",1);
+        if (flags & NOTIFY_ZSET) res = sdscatlen(res,"z",1);
+        if (flags & NOTIFY_EXPIRED) res = sdscatlen(res,"x",1);
+        if (flags & NOTIFY_EVICTED) res = sdscatlen(res,"e",1);
     }
-    if (flags & REDIS_NOTIFY_KEYSPACE) res = sdscatlen(res,"K",1);
-    if (flags & REDIS_NOTIFY_KEYEVENT) res = sdscatlen(res,"E",1);
+    if (flags & NOTIFY_KEYSPACE) res = sdscatlen(res,"K",1);
+    if (flags & NOTIFY_KEYEVENT) res = sdscatlen(res,"E",1);
     return res;
 }
 
@@ -104,25 +104,25 @@ void notifyKeyspaceEvent(int type, char *event, robj *key, int dbid) {
     eventobj = createStringObject(event,strlen(event));
 
     /* __keyspace@<db>__:<key> <event> notifications. */
-    if (server.notify_keyspace_events & REDIS_NOTIFY_KEYSPACE) {
+    if (server.notify_keyspace_events & NOTIFY_KEYSPACE) {
         chan = sdsnewlen("__keyspace@",11);
         len = ll2string(buf,sizeof(buf),dbid);
         chan = sdscatlen(chan, buf, len);
         chan = sdscatlen(chan, "__:", 3);
         chan = sdscatsds(chan, key->ptr);
-        chanobj = createObject(REDIS_STRING, chan);
+        chanobj = createObject(OBJ_STRING, chan);
         pubsubPublishMessage(chanobj, eventobj);
         decrRefCount(chanobj);
     }
 
     /* __keyevente@<db>__:<event> <key> notifications. */
-    if (server.notify_keyspace_events & REDIS_NOTIFY_KEYEVENT) {
+    if (server.notify_keyspace_events & NOTIFY_KEYEVENT) {
         chan = sdsnewlen("__keyevent@",11);
         if (len == -1) len = ll2string(buf,sizeof(buf),dbid);
         chan = sdscatlen(chan, buf, len);
         chan = sdscatlen(chan, "__:", 3);
         chan = sdscatsds(chan, eventobj->ptr);
-        chanobj = createObject(REDIS_STRING, chan);
+        chanobj = createObject(OBJ_STRING, chan);
         pubsubPublishMessage(chanobj, key);
         decrRefCount(chanobj);
     }
